"6809"
	  TITLE     "TNC EPROM Programmer support routines, M. Morrison"

;	  Copyright, (C) Margaret S. Morrison, 1982,1983

;	  Permission is granted for use so long as this notice of
;	  authorship is retained in the source code.

;	  This routine is intended to complement the Version 3 and
;	  later software for the Tucson Amateur Packet Radio Terminal
;	  Node Controller.  It provides for control of the TAPR TNC
;	  EPROM Programmer Attachment.

	  INCLUDE   PARAM
	  INCLUDE   RAMTAB4

	  TITLE     "TNC EPROM Programmer support routines, M. Morrison"

	  PROG
	  DIRECT
	  EXPAND
	  GLOBAL    PROGRM,CALIBR
	  GLOBAL    Monbeg

	  EXTERNAL  WRITE1,WRITECR,WRITEBLK
	  EXTERNAL  Write,Getlin,Wrtbyt,Dehex,Enhex,Wrthex
	  EXTERNAL  Bltrn,Inton,Intoff,Setint,Vinit,Hinit

PROGRM	  LBRA	    PROGRAM
CALIBR	  LBRA	    CALIBR_
Monbeg	  LBRA	    MONBEG		;resume normal output

	  IF	    Beta

PROGRAM   LDY	    #NOTMSG
	  LDD	    #NOTLEN
	  LBSR	    Write
	  RTS

NOTMSG	  ASC	    "Not implemented"
	  FCB	    CR
NOTLEN	  EQU	    ($-NOTMSG)

	  ELSE

; Set programmer for insertion or removal of EPROM.  On return, Port
; A is input, Port B is zero, and the address latched is zero.
; Calls SETADR and so returns with IOPAGE in DP.

	  SETDP     IOPAGE
CLEAR	  LDX	    #0			;get ready for prom insertion
	  BSR	    SETADR		;clear address
	  BSR	    INA			;set A to inputs
	  CLR	    PIB			;turn off controls
	  RTS

; Program high and low bytes of address to latches.  On return,
; programmer is deselected and Port A is output. Sets DP to IOPAGE.

SETADR	  LDA	    #IOPAGE
	  TFR	    A,DP
	  PSHS	    X			;set address to (X)
	  LDB	    PIB
	  BIT	    PGM_+CE_+OE_+ASEL,0
	  BSR	    OUTA
	  PULS	    A			;set high bits
	  STA	    PIA
	  BSR	    CLOCKT
	  BIT	    0,ASEL
	  PULS	    A			;set low bits
	  STA	    PIA
	  BSR	    CLOCKT
	  RTS

; ** This group of routines assume IOPAGE in DP. **

; Read high and low address bytes from latches.  On return, chip
; is in read-address mode and Port A is input.

GETADR	  BSR	    INA			;read address
	  LDB	    PIB
	  BIT	    PGM_+CE_,OE_+ASEL
	  BIT	    CLK,0
	  LDA	    PIA			;get low address
	  PSHS	    A
	  BIT	    ASEL,CLK
	  BIT	    CLK,0
	  LDA	    PIA			;get high address
	  BIT	    0,CLK
	  ANDA	    #3FH		;get rid of status bits
	  PSHS	    A
	  PULS	    X,PC		;return in X

; Set Port A to inputs.

INA	  CLR	    CRA			;command DDRA
	  PSHS	    A,B
	  LDD	    #00004H		;set to inputs, reset command
	  STD	    PDDRA
	  PULS	    A,B,PC

; Set Port A to outputs.

OUTA	  CLR	    CRA			;command DDRA
	  PSHS	    A,B
	  LDD	    #0FF04H		;set to outputs, reset command
	  STD	    PDDRA
	  PULS	    A,B,PC

; Toggle CLK signal to programmer.

CLOCKT	  BIT	    CLK,0		;toggle clock
	  BIT	    0,CLK
	  RTS

; ** End of IOPAGE routines. **

; Show current parameter values

SHOW	  LDU	    #MSGLST		;get prom parameters
	  LDA	    #MSGNUM
	  PSHS	    A			;save counter
SHLOOP	  PULU	    B,Y			;get identifier string
	  CLRA
	  LBSR	    Write		;write it
	  LDD	    [,U++]		;get value
	  LBSR	    Wrthex		;write it
	  LBSR	    WRITECR
	  DEC	    ,S
	  BNE	    SHLOOP		;loop
	  PULS	    A
	  LBRA	    GETOPT

; Get input data after prompts are typed.  Call with list pointer
; in U.

NEWPRM	  BSR	    GETINP		;get prom parameters
	  LBRA	    GETOPT

GETINP	  LDU	    #MSGLST
	  LDA	    #MSGNUM
	  PSHS	    A			;save counter
MSLOOP	  PULU	    B,Y			;get prompt string
	  CLRA
	  LBSR	    Write
	  LEAS	    -20H,S		;make space for buffer
	  TFR	    S,Y
	  LBSR	    Getlin		;prompt for number
	  LBSR	    Dehex
	  LEAS	    20H,S		;clear buffer
	  STD	    [,U++]		;store number
	  DEC	    ,S
	  BNE	    MSLOOP
	  PULS	    A,PC

; Main program entry

	  SETDP     PAGE
PROGRAM   TST	    ABAUD+1
	  BNE	    PROG_0
	  RTS
PROG_0	  INC	    DIA_FL		;don't write junk to parallel port
	  PSHS	    CC,U
	  LDU	    #PRO_CT
	  PULU	    B,X
	  PSHS	    B,X
	  LDB	    #OPTLEN
	  LDX	    #OPTMS
	  PSHU	    B,X
	  LDD	    TTI_IV		;save interrupt vectors
	  STD	    >TTISAV
	  LDD	    CLK_IV
	  STD	    >TIMSAV
	  LDB	    #(TTI+TTO)		;terminal interrupts only
	  LBSR	    Setint

	  LDY	    #HEADER
	  LDD	    #HEADLN
	  LBSR	    Write
	  BSR	    GETINP

SHOOPT	  LDY	    #LISMS		;show whole option list
	  LDD	    #LISLEN		;get option
	  LBSR	    Write

; Turn off everything at end of each option.

	  SETDP     IOPAGE
GETOPT	  PSHS	    DP			;reset prom socket
	  LBSR	    CLEAR		;this alters DP
	  LBSR	    POWER
	  PULS	    DP

	  SETDP     PAGE
	  LDU	    #OPLIST
	  PULU	    B,Y			;get option
	  CLRA
	  LBSR	    Write
	  LEAS	    -20H,S		;make buffer space
	  TFR	    S,Y
	  LBSR	    Getlin
	  PULU	    B,X			;decode option
	  LDA	    ,Y
	  ANDA	    #137Q		;upper case
	  LEAS	    20H,S		;clear buffer space

OPLOOP	  CMPA	    ,-X			;find match
	  BEQ	    OPTION
	  DECB
	  BGE	    OPLOOP		;B contains option number
	  BRA	    SHOOPT

OPTION	  LSLB				;jump to routine
	  LDX	    #DESTAB
	  JMP	    [B,X]

DESTAB	  FDB	    BLANKC,CHECKP,LISTRM,NEWPRM,PRGRAM,RETURN
	  FDB	    SHOW,TRANSF,VERIFY

; Read data and set up registers, set up prom socket.
; Returns with IOPAGE in DP and PIB in B.

	  SETDP     PAGE
GETRNG	  LDD	    >DESTIN		;set up registers
	  SUBD	    >BASADR
	  TFR	    D,X
	  LDY	    >SOURCE
	  LDU	    >BYTCNT

	  SETDP     IOPAGE
SETRNG	  LBSR	    SETADR		;set address in programmer
	  LBSR	    INA			;changes DP

	  BIT	    PGM_+VCC,OE_+CE_	;turn on power, read data mode
	  LBSR	    POWER		;wait
	  LDB	    PIB
	  RTS

; Blank check.	Types first failed address (offset).

	  SETDP     PAGE
BLANKC	  PSHS	    DP
	  LDY	    #BMSG		;type blank message
	  LDD	    #BLEN
	  LBSR	    Write
	  BSR	    GETRNG
	  TFR	    U,Y			;count in Y

	  SETDP     IOPAGE
BLK_LP	  LDA	    PIA			;get data
	  CMPA	    #0FFH		;check blank
	  BNE	    EROR_B
	  LBSR	    CLOCKT		;clock address
	  LEAY	    -1,Y
	  BNE	    BLK_LP

	  SETDP     PAGE
	  PULS	    DP
	  LDY	    #BOKMS
	  LDD	    #BOKLEN
	  LBSR	    Write
	  LBRA	    GETOPT

EROR_B	  LBSR	    GETADR		;report 1st non-blank addr

	  PULS	    DP
	  PSHS	    X			;save address
	  LDY	    #BNOKMS		;write message
	  LDD	    #BNOKLN
	  LBSR	    Write
	  LDD	    >BASADR
	  ADDD	    ,S++		;write address
	  LBSR	    Wrthex
	  LBSR	    WRITECR
	  LBRA	    GETOPT

; List contents of ROM, 16 bytes per line, with offset address on
; each line.

LISTRM	  PSHS	    DP
	  LDY	    #LMSG		;type list message
	  LDD	    #LLEN
	  LBSR	    Write
	  LBSR	    GETRNG

	  PULS	    DP
	  LDD	    >DESTIN
	  PSHS	    D			;save address on stack
	  PSHS	    A			;make space for counter

LISLP2	  LBSR	    WRITECR		;type address
	  LDD	    1,S
	  LBSR	    Wrthex
	  LDD	    #10H		;set line counter
	  STB	    ,S
	  ADDD	    1,S			;update address
	  STD	    1,S

LISLP1	  LEAS	    -2,S		;make space
	  TFR	    S,Y			;for hex string
	  BSR	    RDBYTE		;read data
	  LBSR	    Enhex		;encode it
	  LDA	    #" "		;with blank preceding
	  PSHS	    A
	  TFR	    S,Y			;write out string
	  LDD	    #3
	  LBSR	    Write
	  LEAS	    3,S			;fix stack
	  LEAU	    -1,U		;count bytes
	  DEC	    ,S			;count line counter
	  BNE	    LISLP1

	  CMPU	    #0			;check byte count
	  BGT	    LISLP2

	  LEAS	    3,S			;address & counter off
	  LBSR	    WRITECR
	  LBRA	    GETOPT

	  SETDP     IOPAGE
RDBYTE	  PSHS	    B,DP
	  LDA	    #IOPAGE
	  TFR	    A,DP
	  LDB	    PIB
	  LDA	    PIA			;get data
	  LBSR	    CLOCKT		;clock address
	  PULS	    B,DP,PC

	  SETDP     PAGE
TRANSF	  PSHS	    DP
	  LDY	    #TMSG		;type transfer message
	  LDD	    #TLEN
	  LBSR	    Write
	  LBSR	    GETRNG		;set up prom
	  PULS	    DP

	  PSHS	    U
	  LDD	    >HITOP		;look for space
	  SUBD	    >HIBOT
	  BNE	    TRAN_0

	  LDY	    #TNOKMS		;no ram message
	  LDD	    #TNOKLN
	  LBSR	    Write
	  LEAS	    2,S
	  LBRA	    GETOPT

TRAN_0	  CMPD	    ,S			;get RAM size or count
	  BGE	    TRAN_a
	  STD	    ,S
TRAN_a	  LDY	    ,S			;get byte count in Y
	  LDX	    >HIBOT		;RAM destination

TRAN_1	  BSR	    RDBYTE		;read data
	  STA	    ,X+
	  LEAY	    -1,Y
	  BNE	    TRAN_1

	  LDD	    #TOKLN
	  LDY	    #TOKMS
	  LBSR	    Write
	  PULS	    D
	  LBSR	    Wrthex
	  LBSR	    WRITECR
	  LBRA	    GETOPT

; Check programmer function.

CHECKP	  PSHS	    DP
	  LDY	    #CMSG		;type check message
	  LDD	    #CLEN
	  LBSR	    Write
	  LDX	    #0
	  LBSR	    SETRNG

	  SETDP     IOPAGE
CHEKLP	  PSHS	    X			;address reference
	  LBSR	    GETADR		;read address
	  CMPX	    ,S			;check correct value
	  BNE	    EROR_C
	  BIT	    0,CE_+OE_		;increment counter
	  LBSR	    CLOCKT
	  PULS	    X
	  LEAX	    1,X			;increment ref. address
	  CMPX	    >BYTCNT		;compare with final
	  BNE	    CHEKLP

	  SETDP     PAGE
	  PULS	    DP
	  LDY	    #COKMS		;write OK message
	  LDD	    #COKLN
	  LBSR	    Write
	  LBRA	    GETOPT

EROR_C	  PULS	    D,DP		;fix DP, save address
	  PSHS	    D
	  LDY	    #CNOKMS		;write not OK message
	  LDD	    #CNOKLN
	  LBSR	    Write
	  PULS	    D
	  LBSR	    Wrthex		;write address for fail
	  LBSR	    WRITECR		; (absolute address)
	  LBRA	    GETOPT

; Read data from EPROM.  Assumes entry with Port A output and
; returns to this state.  Preserves Port B.  Requires IOPAGE in DP.

	  SETDP     IOPAGE
GETDAT	  PSHS	    B			;read data register
	  LBSR	    INA
	  BIT	    PGM_,OE_+CE_	;select programmer
	  LDA	    PIA
	  PULS	    B			;restore B
	  STB	    PIB			;restore PIB
	  LBSR	    OUTA
	  RTS

; Program EPROM.  Typein during programming aborts procedure.

	  SETDP     PAGE
PRGRAM	  LDX	    CLK_IV
	  PSHS	    DP,X
	  LDY	    #PMSG		;type program message
	  LDD	    #PLEN
	  LBSR	    Write

	  SETDP     IOPAGE
	  LBSR	    GETRNG		;set other voltages
					;now have IOPAGE in DP
	  BIT	    OE_+VCC+VPP+VHI,0	;turn on programming voltage
	  LBSR	    POWER		;wait
	  SEI
	  LDB	    #(TIM+TTO+TTI)	;terminal + timer
	  LBSR	    Setint
	  LDB	    PIB
	  LDX	    #CANCEL		;typein cancels programming
	  STX	    >TTI_IV
	  LBSR	    OUTA		;output mode

; Loop to program EPROM.  This is Intel fast algorithm.
; Interrupts off except at CWAI instructions

DAT_LP	  CLR	    ,-S			;zero counter ***
	  LDA	    ,Y			;get data byte
	  STA	    PIA
	  BIT	    OE_,PGM_+CE_	;write to data bus

	  LDX	    #T_1MS		;start timer
	  STX	    TIM2
	  LDX	    #TIME_1		;set interrupt destination
	  STX	    >CLK_IV

; Check data at most 15 times at 1-ms intervals

BRN_LP	  LDA	    ,S			;check count
	  CMPA	    #0FH
	  BEQ	    FNL_WT		;done
	  INC	    ,S			;increment
CNT_LP	  CWAI	    #0EFH		;wait for timeout
	  BRA	    CNT_LP

TIME_1	  LDD	    TIM2		;restart timer
	  DECA
	  ADDA	    #T_1MSL
	  ADCB	    #T_1MSH
	  STD	    TIM2

	  LEAS	    1,S			;don't enable interrupts
	  PULS	    A,B,DP,X,Y,U
	  LEAS	    2,S			;clear interrupt stack
	  BSR	    GETDAT		;check byte
	  CMPA	    ,Y
	  BNE	    BRN_LP		;no match

; Byte has been burned or 15 ms have elapsed.  Perform final burn.

FNL_WT	  LDX	    #TIME_2		;change timer
	  STX	    >CLK_IV
	  LDX	    #T_4MS
	  STX	    TIM2

FNL_LP	  CWAI	    #0EFH		;wait
	  BRA	    FNL_LP

TIME_2	  LDD	    TIM2		;restart timer
	  DECA
	  ADDA	    #T_4MSL
	  ADCB	    #T_4MSH
	  STD	    TIM2

	  LEAS	    1,S			;don't enable interrupts
	  PULS	    A,B,DP,X,Y,U
	  LEAS	    2,S			;clear interrupt stack
	  DEC	    ,S			;wait loop
	  BNE	    FNL_LP

; Final burn done -- if it doesn't match now, it never will.

	  LEAS	    1,S			;clear counter ***
	  BIT	    PGM_,0		;end programming pulse
	  LBSR	    GETDAT		;check final value
	  CMPA	    ,Y+
	  BNE	    EROR_P		;no match -- prom bad

	  LBSR	    CLOCKT		;clock address
	  LEAU	    -1,U		;check byte count
	  CMPU	    #0
	  BNE	    DAT_LP		;continue with next byte

; Ready for final verify with VCC at +5 volts

	  BIT	    PGM_+OE_+CE_,VPP+VHI ; turn off programming voltage
	  LDX	    >TTISAV		;restore terminal interrupt
	  STX	    >TTI_IV
	  LDB	    #(TTI+TTO)		;terminal only now
	  LBSR	    Setint
	  PULS	    DP,X

	  SETDP     PAGE
	  STX	    CLK_IV
	  CLI
	  BRA	    VERIFY		;automatic verify

; Abort programming here

	  SETDP     IOPAGE
EROR_P	  LDY	    #PNOKMS
	  BRA	    EROR_A

CANCEL	  LEAS	    1,S			;clear stack then same as error
	  PULS	    A,B,DP,X,Y,U
	  LEAS	    3,S			;clear PC and counter ***
	  LDY	    #AMSG

EROR_A	  BIT	    PGM_+OE_+CE_,VPP+VHI ;turn off programming voltage

	  SETDP     PAGE
	  TFR	    DP,A
	  PULS	    DP,X
	  STX	    CLK_IV		;restare clock interrupt
	  LDX	    >TTISAV		;restore terminal interrupt
	  STX	    TTI_IV
	  PSHS	    DP
	  TFR	    A,DP

	  SETDP     IOPAGE
	  LDB	    #(TTI+TTO)		;terminal only now
	  LBSR	    Setint
	  CLI
	  LBSR	    GETADR
	  PULS	    DP

	  SETDP     PAGE
	  PSHS	    X			;save address of failure
	  LDD	    #PNOKLN		;type appropriate message
	  LBSR	    Write
	  LDD	    >BASADR		;type address
	  ADDD	    ,S++
	  LBSR	    Wrthex
	  LBSR	    WRITECR
	  LBRA	    GETOPT

VERIFY	  PSHS	    DP
	  LDY	    #VMSG		;type program message
	  LDD	    #VLEN
	  LBSR	    Write
	  LBSR	    GETRNG

	  SETDP     IOPAGE
VFY_LP	  LDA	    PIA			;get data
	  CMPA	    ,Y+
	  BNE	    EROR_V
	  LBSR	    CLOCKT		;clock address
	  LEAU	    -1,U
	  CMPU	    #0
	  BNE	    VFY_LP

	  SETDP     PAGE
	  PULS	    DP
	  LDY	    #VOKMS
	  LDD	    #VOKLN
	  LBSR	    Write
	  LBRA	    GETOPT

EROR_V	  LBSR	    GETADR
	  PULS	    DP
	  PSHS	    X
	  LDY	    #VNOKMS
	  LDD	    #VNOKLN
	  LBSR	    Write
	  LDD	    >BASADR
	  ADDD	    ,S++
	  LBSR	    Wrthex
	  LBSR	    WRITECR
	  LBRA	    GETOPT

	  SETDP     PAGE
RETURN	  SEI
	  LDX	    #DONE
	  STX	    TTI_IV
	  CLI
	  LDY	    #RMSG
	  LDD	    #RLEN
	  LBSR	    Write
	  LDY	    #DMSG
	  LDD	    #DLEN
	  LBSR	    Write

DON_WT	  CWAI	    #0EFH
	  BRA	    DON_WT

DONE	  LEAS	    1,S			;clear stack
	  PULS	    A,B,DP,X,Y,U
	  LEAS	    2,S
	  LDD	    >TTISAV		;restore interrupt addresses
	  STD	    TTI_IV
	  LDD	    >TIMSAV
	  STD	    CLK_IV

	  LDX	    #T2DAT		;put timer back together
	  STX	    >TIM2
	  LDB	    #(TTI+TTO+TIM+PAK)	;turn all on
	  LBSR	    Setint
	  PULS	    B,Y
	  LDU	    #PRO_CT+3
	  PSHU	    B,Y
	  PULS	    CC,U
	  CLR	    DIA_FL
	  RTS


; Wait routine to let power changes settle down -- about 100 ms.
; Requires IOPAGE in DP

	  SETDP     IOPAGE
POWER	  PSHS	    CC,D,X
	  SEI
	  LDB	    >INTRPT		;save interrupt state
	  LDX	    >CLK_IV
	  PSHS	    B,X
	  LDB	    #(TIM+TTO)		;timer only (let typing
	  LBSR	    Setint		;continue
	  LDD	    #TIME_P
	  STD	    >CLK_IV
	  LDD	    #T2DAT
	  STD	    TIM2
	  LDA	    #0AH		;count 10 10-ms intervals

PWR_WT	  CWAI	    #0EFH		;wait for timer change
	  BRA	    PWR_WT

TIME_P	  LDD	    TIM2		;restart timer
	  DECA
	  ADDA	    #T2DATA
	  ADCB	    #T2DATB
	  STD	    TIM2
	  LEAS	    1,S			;stay disabled
	  PULS	    A,B,DP,X,Y,U
	  LEAS	    2,S			;clear interrupt stack
	  DECA
	  BNE	    PWR_WT
	  PULS	    B,X			;restore interrupt state
	  STX	    >CLK_IV
	  LBSR	    Setint
	  PULS	    CC,D,X,PC

; Message strings

	  COMN
LISMS	  FCB	    CR
	  ASC	    "Options:"
	  FCB	    CR,CR
BMSG	  ASC	    "Blank check"
BLEN	  EQU	    ($-BMSG)
	  FCB	    CR
CMSG	  ASC	    "Check programmer"
CLEN	  EQU	    ($-CMSG)
	  FCB	    CR
LMSG	  ASC	    "List ROM"
	  FCB	    CR
LLEN	  EQU	    ($-LMSG)
	  ASC	    "New prom parameters"
	  FCB	    CR
PMSG	  ASC	    "Program"
	  FCB	    CR
PLEN	  EQU	    ($-PMSG)
SMSG	  ASC	    "Show parameters"
	  FCB	    CR
SLEN	  EQU	    ($-SMSG)
TMSG	  ASC	    "Transfer to RAM"
TLEN	  EQU	    ($-TMSG)
	  FCB	    CR
VMSG	  ASC	    "Verify"
VLEN	  EQU	    ($-VMSG)
	  FCB	    CR
RMSG	  ASC	    "Return"
	  FCB	    CR
RLEN	  EQU	    ($-RMSG)
	  FCB	    CR
LISLEN	  EQU	    ($-LISMS)
OPTMS	  ASC	    "Option: "
OPTLEN	  EQU	    ($-OPTMS)

OPTLIS	  ASC	    "BCLNPRSTV"
OPTCT	  EQU	    ($-OPTLIS)

MSGLST	  FCB	    BASLEN		;get base address
	  FDB	    BASMS
	  FDB	    BASADR
	  FCB	    SRCLEN		;input numbers
	  FDB	    SRCMS
	  FDB	    SOURCE
	  FCB	    DSTLEN
	  FDB	    DSTMS
	  FDB	    DESTIN
	  FCB	    BYTLEN
	  FDB	    BYTMS
	  FDB	    BYTCNT
MSGNUM	  EQU	    ($-MSGLST)/5

OPLIST	  FCB	    OPTLEN
	  FDB	    OPTMS
	  FCB	    OPTCT-1		;option codes
	  FDB	    (OPTLIS+OPTCT)

VNOKMS
CNOKMS
BNOKMS	  ASC	    " fail at "
BNOKLN	  EQU	    ($-BNOKMS)
CNOKLN	  EQU	    ($-CNOKMS)
VNOKLN	  EQU	    ($-VNOKMS)

AMSG	  ASC	    " aborted at "
PNOKMS	  ASC	    " failure at "
PNOKLN	  EQU	    ($-PNOKMS)
TNOKMS	  ASC	    " failure: no RAM"
	  FCB	    CR
TNOKLN	  EQU	    ($-TNOKMS)

COKMS
BOKMS
VOKMS	  ASC	    " OK"
	  FCB	    CR
VOKLN	  EQU	    ($-VOKMS)
BOKLEN	  EQU	    ($-BOKMS)
COKLN	  EQU	    ($-COKMS)

TOKMS	  ASC	    " OK: "
TOKLN	  EQU	    ($-TOKMS)

DMSG	  ASC	    "Remove prom & type any character"
	  FCB	    CR
DLEN	  EQU	    ($-DMSG)

HEADER	  ASC	    "TAPR TNC EPROM PROGRAMMER"
	  FCB	    CR,CR
HEADLN	  EQU	    ($-HEADER)
BASMS	  ASC	    "Prom base address: "
BASLEN	  EQU	    ($-BASMS)

SRCMS	  ASC	    "Source (TNC) address: "
SRCLEN	  EQU	    ($-SRCMS)

DSTMS	  ASC	    "Destination (EPROM) address: "
DSTLEN	  EQU	    ($-DSTMS)

BYTMS	  ASC	    "Byte count: "
BYTLEN	  EQU	    ($-BYTMS)

	  ENDIF
	  TITLE     "Modem Calibration -- TNC LLR, Margaret Morrison"

	  PROG
	  SETDP     PAGE
CALIBR_   LDX	    TTI_IV		;save interrupt addresses
	  LDY	    CLK_IV
	  PSHS	    X,Y,U
	  SEI				;disable interrupt
	  LDB	    #(TIM+PAK)		;turn off timer & HDLC
	  LBSR	    Intoff		;page ok

	  LDB	    #CALMOD		;reset timer mode
	  STB	    >AUXCR
	  LDA	    >SR			;turn off tone/set low tone
	  ANDA	    #MISCIN
	  LSLA
	  PSHS	    A

	  IF	    Beta		;different options for Beta & regular
	  LDA	    #0CCH
	  ORA	    ,S+

	  ELSE
	  LDA	    #0ECH
	  EORA	    ,S+

	  ENDIF

	  STA	    >BOTHOF		;save low tone data
	  STA	    >PERCR

	  CLI				;get page back
	  LDU	    #PRO_CT		;save prompt string & replace with new one
	  PULU	    B,Y
	  PSHS	    B,Y
	  LDY	    #CALPR
	  LDB	    #CLPLN
	  PSHU	    B,Y
	  LEAS	    -20H,S		;make space for buffer

CALI_2	  LDY	    #CALMS1		;main calibration message
	  LDD	    #CLEN1
	  LBSR	    Write

CALMNU	  LDY	    #CALPR
	  LDD	    #CLPLN
	  LBSR	    Write		;prompt for line
	  TFR	    S,Y			;get line in temp. buffer
	  LBSR	    Getlin
	  LDX	    #CALTAB
	  LDA	    ,Y+			;return character

	  SUBA	    #61Q
	  BLT	    CALI_2		;error -- type menu
	  CMPA	    #5
	  BGT	    CALI_2
	  LSLA
	  JMP	    [A,X]		;jump to appropriate routine

	  COMN
CALTAB	  FDB	    SETLO,SETHI,SETMD,CWCAL,ROMCHK,STPCAL

	  PROG
WRTLST	  PSHS	    A			;subroutine to write list
WRTL_1	  PULU	    B,Y
	  CLRA
	  LBSR	    Write
	  DEC	    ,S
	  BNE	    WRTL_1
	  PULS	    A,PC

STPCAL	  LDU	    #STLIST		;Reset message
	  LDA	    #STNUM
	  BSR	    WRTLST
	  LDX	    #RESETC

STPC_2	  SEI
	  STX	    TTI_IV
STPC_3	  CWAI	    #0EFH
	  BRA	    STPC_3

RESETC	  LEAS	    1,S			;leave disabled
	  PULS	    A,B,DP,X,Y,U	;get the rest back
	  LEAS	    22H,S		;but not the PC -- clear buffers
	  PULS	    B,Y			;restore prompt strings
	  LDU	    #PRO_CT+3
	  PSHU	    B,Y
	  PULS	    X,Y,U
	  STY	    CLK_IV		;restore interrupt vectors
	  STX	    TTI_IV
	  LBSR	    Vinit		;restart HDLC
	  LBSR	    Hinit
	  LDB	    #(TTI+TTO+PAK+TIM)	;turn on everything
	  LBSR	    Setint
	  CLI				;clear interrupts
	  RTS

CWCAL	  LDU	    #CWLIST		;Tone calibrate message
	  LDA	    #CWNUM
	  BSR	    WRTLST
	  SEI
	  LBSR	    Vinit
	  LBSR	    Hinit
	  LDB	    #(TTI+TTO)
	  LBSR	    Setint
	  LDX	    #TOGGLE		;divert typein interrupt
	  BRA	    STPC_2

SETLO	  PSHS	    Y
	  LDU	    #LOLIST		;1200 hz message
	  LDA	    #LONUM
	  BSR	    WRTLST
	  PULS	    Y
	  LDX	    #DATA12
	  BRA	    START

SETHI	  PSHS	    Y
	  LDU	    #HILIST		;2200 hz message
	  LDA	    #HINUM
	  BSR	    WRTLST
	  PULS	    Y
	  LDX	    #DATA22
	  BRA	    START

SETMD	  PSHS	    Y
	  LDU	    #MDLIST		;1700 hz message
	  LDA	    #MDNUM
	  LBSR	    WRTLST
	  PULS	    Y
	  LDX	    #DATA17

; Calibration interrupt service routines

START	  LDA	    ,Y+			;get input frequency data
	  CMPA	    #"/"		;start input number
	  BNE	    STAR_1
	  LDA	    ,Y+
	  CMPA	    #"$"
	  BNE	    STAR_a
	  LBSR	    Dehex		;get number
	  BRA	    STAR_b
STAR_a	  LEAY	    -1,Y
	  LBSR	    DECMAL
STAR_b	  ADDD	    #0FFAH		;correct & reverse
	  EXG	    A,B
	  TFR	    D,X
STAR_1	  SEI				;set up for calibration
	  LDB	    #TIM		;turn on timer
	  LBSR	    Inton		;page ok
	  LDD	    #QUITCL		;input interrupt to halt
	  STD	    TTI_IV
	  LDD	    #TIME1		;start sync on timer 2
	  STD	    CLK_IV
	  LDY	    #100H		;start timer 2
	  STY	    >TIM2		;exit with data for timers ready
STAR_2	  CWAI	    #0EFH
	  BRA	    STAR_2

	  SETDP     PAGE
QUITCL	  TFR	    B,DP
	  LDB	    #TIM		;stop calibration on any typein
	  LBSR	    Intoff		;disable timer interrupt
					;page ok
	  LEAS	    12,S		;adjust stack
	  LDX	    23H,S		;restore terminal interrupt
	  STX	    TTI_IV
	  CLI

	  LDY	    #OFMSG1		;report number to user
	  LDD	    #OFLEN1
	  LBSR	    Write
	  LDD	    #1000H
	  SUBD	    TCOUNT		;get error in count
	  BGE	    QUIT_1
	  NEGA
	  NEGB
	  SBCA	    #0
	  PSHS	    D
	  LDA	    #"-"		;add minus sign
	  LBSR	    WRITE1
	  PULS	    D
QUIT_1	  LBSR	    Wrthex
	  LDY	    #OFMSG2		;finish message
	  LDD	    #OFLEN2
	  LBSR	    Write
	  LBRA	    CALMNU		;back to prompt

	  SETDP     IOPAGE
TIME1	  LDX	    4,S
	  LDY	    6,S
	  STX	    TIM1		;start timer 1 on timer 2
	  STY	    TIM2		;restart timer 2
	  LDD	    #TIME2
	  STD	    >CLK_IV
	  LDA	    >BOTHOF		;turn off both lights
	  STA	    PERCR
	  RTI

TIME2	  LDX	    4,S
	  LDY	    6,S
	  LDD	    TIM1		;get timer 1 count
	  STX	    TIM1
	  STY	    TIM2		;restart timers
	  EXG	    A,B
	  DECB
	  STD	    >TCOUNT
	  CMPD	    #1000H
	  BLT	    TIMLO		;too many cts -- freq low
	  BGT	    TIMHI
	  LDA	    #022H		;exact -- both lights
	  BRA	    TIMALL
TIMLO	  LDA	    #020H		;low -- low light
	  BRA	    TIMALL
TIMHI	  LDA	    #002H		;high -- high light
TIMALL	  EORA	    >BOTHOF
	  STA	    PERCR
	  RTI

TOGGLE	  ANDA	    #0DFH		;examine character
	  CMPA	    #"Q"
	  BNE	    TOGG_1		;exit this routine

	  LDA	    #0FFH		;unkey transmitter
	  STA	    >CR1
	  LDA	    >BOTHOF		;turn off tone, etc.
	  STA	    PERCR
	  LDA	    #CALMOD		;reset timer mode for calibration
	  STA	    AUXCR

	  LEAS	    12,S		;adjust stack
	  LDX	    23H,S		;restore terminal interrupt

	  SETDP     PAGE
	  TFR	    B,DP		;get PAGE in DP at DISPAT
	  STX	    TTI_IV
	  CLI
	  LBRA	    CALMNU		;back to prompt

	  SETDP     IOPAGE
TOGG_1	  CMPA	    #"K"
	  BNE	    TOGG_2
	  LDA	    >CR1		;key / unkey transmitter
	  EORA	    #1
	  STA	    >CR1
	  RTI

TOGG_2	  LDA	    PERCR		;toggle tone value
	  EORA	    #20H
	  STA	    PERCR
	  RTI

	  SETDP     PAGE
ROMCHK	  LDY	    #CKMS1
	  LDD	    #CKLN1
	  LBSR	    Write

	  LDX	    #08000H
	  BSR	    CHKSUM
	  BSR	    CHKSUM
	  BSR	    CHKSUM
	  BSR	    CHKSUM
	  LBSR	    WRITECR
	  LBRA	    CALMNU

CHKSUM	  TFR	    X,D
	  LBSR	    Wrthex
	  LBSR	    WRITEBLK
	  LDY	    #2000H
	  CLRA
	  CLRB
CHKS_1	  ADDB	    ,X+			;add to 16 bit total
	  ADCA	    #0
	  LEAY	    -1,Y
	  BNE	    CHKS_1
	  LBSR	    Wrthex
	  LBSR	    WRITECR
	  RTS

; Decode decimal input number
; Input   Y   pointer to input string
; Output  Y   updated pointer
;	  D   result
; Overflow returns large + number

DECMAL	  CLRA
	  CLRB
	  PSHS	    A,B			;leave space for answer
DECM_1	  LDB	    ,Y+			;get next character
	  SUBB	    #"0"
	  BLT	    DECM_2
	  CMPB	    #9
	  BGT	    DECM_2
	  PSHS	    B			;valid next character
	  LDA	    1,S			;multiply high half
	  LDB	    #0AH
	  MUL
	  TSTA
	  BNE	    DECM_3		;overrun
	  STB	    1,S			;store high half
	  LDA	    2,S			;multiply low half
	  LDB	    #0AH
	  MUL
	  ADDB	    ,S+			;add new digit & remove
	  ADCA	    ,S			;add high byte
	  BCS	    DECM_4		;overrun
	  STD	    ,S			;save result
	  BRA	    DECM_1

DECM_2	  PULS	    D,PC

DECM_3	  LEAS	    1,S			;overrun -> + infinity
DECM_4	  LEAS	    2,S
	  LDD	    #7FFFH
	  RTS

	  COMN
	  IF	    Beta		;messages for BETA version
CALMS1	  FCB	    CR
	  ASC	    "	  Beta TNC Calibration and Checkout"
	  FCB	    CR,CR
	  ASC	    "Menu:"
LOMS1	  FCB	    CR
	  ASC	    "	1 Set space tone (1200 hz) - 5th pot"
LOLN1	  EQU	    ($-LOMS1)
HIMS1	  FCB	    CR
	  ASC	    "	2 Set mark tone (2200 hz) - 4th pot"
HILN1	  EQU	    ($-HIMS1)
MDMS1	  FCB	    CR
	  ASC	    "	3 Set demodulator (1700 hz) - top pot"
MDLN1	  EQU	    ($-MDMS1)
CWMS1	  FCB	    CR
	  ASC	    "	4 Set tone null - 2nd pot"
CWLN1	  EQU	    ($-CWMS1)
CKMS1	  FCB	    CR
	  ASC	    "	5 ROM checksum"
STMS1	  FCB	    CR
CKLN1	  EQU	    ($-CKMS1)
	  ASC	    "	6 End calibration and reset"
	  FCB	    CR
STLN1	  EQU	    ($-STMS1)
	  FCB	    CR
CLEN1	  EQU	    ($-CALMS1)

LOMS2	  ASC	    "JP4,JP7,JP11"
LOLN2	  EQU	    ($-LOMS2)

LOMS3	  ASC	    "s JP6,JP10"
LOLN3	  EQU	    ($-LOMS3)

LOMS4	  ASC	    "29"
LOLN4	  EQU	    ($-LOMS4)

HIMS2	  ASC	    "JP4,JP7,JP10,JP11"
HILN2	  EQU	    ($-HIMS2)

HIMS3	  ASC	    " JP6"
HILN3	  EQU	    ($-HIMS3)

HIMS4	  ASC	    "27"
HILN4	  EQU	    ($-HIMS4)

MDMS2	  ASC	    "JP2,JP5,JP6,JP9"
MDLN2	  EQU	    ($-MDMS2)

MDMS3	  ASC	    "s JP4,JP7,JP8"
MDLN3	  EQU	    ($-MDMS3)

MDMS4	  ASC	    "16"
MDLN4	  EQU	    ($-MDMS4)

STMS2	  ASC	    "JP4,JP6,JP7,JP8,JP10"
STLN2	  EQU	    ($-STMS2)

STMS3	  ASC	    "s JP2,JP3,JP5,JP11"
	  FCB	    CR
	  ASC	    "JP9 depends on radio"
STLN3	  EQU	    ($-STMS3)

CWMS2	  ASC	    "33"
	  FCB	    CR
	  ASC	    "Type 'Q' to quit, 'K' to key/unkey transmitter,"
	  FCB	    CR
	  ASC	    "any other key to toggle tone."
	  FCB	    CR
CWLN2	  EQU	    ($-CWMS2)


	  ELSE				;messages for GAMMA version
CALMS1	  FCB	    CR
	  ASC	    "	  TAPR TNC Calibration and Checkout"
	  FCB	    CR,CR
	  ASC	    "Menu:"
LOMS1	  FCB	    CR
	  ASC	    "	1 Set low tone (1200 hz) - 1st pot"
LOLN1	  EQU	    ($-LOMS1)
HIMS1	  FCB	    CR
	  ASC	    "	2 Set high tone (2200 hz) - 2nd pot"
HILN1	  EQU	    ($-HIMS1)
MDMS1	  FCB	    CR
	  ASC	    "	3 Set demodulator (1700 hz) - 5th pot"
MDLN1	  EQU	    ($-MDMS1)
CWMS1	  FCB	    CR
	  ASC	    "	4 Set audio level - 4th pot"
	  FCB	    CR
	  ASC	    "	  and tone null - 3rd pot"
CWLN1	  EQU	    ($-CWMS1)
CKMS1	  FCB	    CR
	  ASC	    "	5 ROM checksum"
STMS1	  FCB	    CR
CKLN1	  EQU	    ($-CKMS1)
	  ASC	    "	6 End calibration and reset"
	  FCB	    CR
STLN1	  EQU	    ($-STMS1)
	  FCB	    CR
CLEN1	  EQU	    ($-CALMS1)

LOMS2	  ASC	    "JP1,JP3,JP4"
LOLN2	  EQU	    ($-LOMS2)

LOMS3	  ASC	    "s JP2,JP5"
LOLN3	  EQU	    ($-LOMS3)

LOMS4	  ASC	    "38"
LOLN4	  EQU	    ($-LOMS4)

HIMS2	  ASC	    "JP3,JP4,JP5"
HILN2	  EQU	    ($-HIMS2)

HIMS3	  ASC	    " JP2"
HILN3	  EQU	    ($-HIMS3)

HIMS4	  ASC	    "36"
HILN4	  EQU	    ($-HIMS4)

MDMS2	  ASC	    "JP1,JP2"
MDLN2	  EQU	    ($-MDMS2)

MDMS3	  ASC	    " JP3"
MDLN3	  EQU	    ($-MDMS3)

MDMS4	  ASC	    "43"
MDLN4	  EQU	    ($-MDMS4)

STMS2	  ASC	    "JP2,JP3,JP5"
STLN2	  EQU	    ($-STMS2)

STMS3	  ASC	    "s JP1,JP4"
STLN3	  EQU	    ($-STMS3)

CWMS2	  ASC	    "31 and R33"
	  FCB	    CR
	  ASC	    "Type 'Q' to quit, 'K' to key/unkey transmitter,"
	  FCB	    CR
	  ASC	    "any other key to switch tone."
	  FCB	    CR,CR
CWLN2	  EQU	    ($-CWMS2)

	  ENDIF

CALPR	  ASC	    "cal:"
CLPLN	  EQU	    ($-CALPR)

LOLIST	  FCB	    LOLN1
	  FDB	    LOMS1
	  FCB	    CLEN2
	  FDB	    CALMS2
	  FCB	    LOLN2
	  FDB	    LOMS2
	  FCB	    CLEN3
	  FDB	    CALMS3
	  FCB	    LOLN3
	  FDB	    LOMS3
	  FCB	    CLEN5
	  FDB	    CALMS5
	  FCB	    LOLN4
	  FDB	    LOMS4
	  FCB	    CLEN4
	  FDB	    CALMS4
LONUM	  EQU	    ($-LOLIST)/3

HILIST	  FCB	    HILN1
	  FDB	    HIMS1
	  FCB	    CLEN2
	  FDB	    CALMS2
	  FCB	    HILN2
	  FDB	    HIMS2
	  FCB	    CLEN3
	  FDB	    CALMS3
	  FCB	    HILN3
	  FDB	    HIMS3
	  FCB	    CLEN5
	  FDB	    CALMS5
	  FCB	    HILN4
	  FDB	    HIMS4
	  FCB	    CLEN4
	  FDB	    CALMS4
HINUM	  EQU	    ($-HILIST)/3

MDLIST	  FCB	    MDLN1
	  FDB	    MDMS1
	  FCB	    CLEN2
	  FDB	    CALMS2
	  FCB	    MDLN2
	  FDB	    MDMS2
	  FCB	    CLEN3
	  FDB	    CALMS3
	  FCB	    MDLN3
	  FDB	    MDMS3
	  FCB	    CLEN5
	  FDB	    CALMS5
	  FCB	    MDLN4
	  FDB	    MDMS4
	  FCB	    CLEN4
	  FDB	    CALMS4
MDNUM	  EQU	    ($-MDLIST)/3

STLIST	  FCB	    STLN1
	  FDB	    STMS1
	  FCB	    CLEN2
	  FDB	    CALMS2
	  FCB	    STLN2
	  FDB	    STMS2
	  FCB	    CLEN3
	  FDB	    CALMS3
	  FCB	    STLN3
	  FDB	    STMS3
	  FCB	    CLEN4
	  FDB	    CALMS4
STNUM	  EQU	    ($-STLIST)/3

CWLIST	  FCB	    CWLN1
	  FDB	    CWMS1
	  FCB	    CLEN2
	  FDB	    CALMS2
	  FCB	    STLN2
	  FDB	    STMS2
	  FCB	    CLEN3
	  FDB	    CALMS3
	  FCB	    STLN3
	  FDB	    STMS3
	  FCB	    CLEN5
	  FDB	    CALMS5
	  FCB	    CWLN2
	  FDB	    CWMS2
CWNUM	  EQU	    ($-CWLIST)/3

CALMS2	  FCB	    CR
	  ASC	    "Remove jumpers "
CLEN2	  EQU	    ($-CALMS2)

CALMS3	  FCB	    CR
	  ASC	    "Place jumper"
CLEN3	  EQU	    ($-CALMS3)

CALMS4	  FCB	    CR
	  ASC	    "Type any key to continue"
	  FCB	    CR,CR
CLEN4	  EQU	    ($-CALMS4)

CALMS5	  FCB	    CR
	  ASC	    "Adjust R"
CLEN5	  EQU	    ($-CALMS5)

OFMSG1	  ASC	    "Error is "
OFLEN1	  EQU	    ($-OFMSG1)

OFMSG2	  ASC	    " counts."
	  FCB	    CR,CR
OFLEN2	  EQU	    ($-OFMSG2)

	  IF	    Beta

MONBEG	  RTI

	  ELSE

	  TITLE     "Debugging Monitor -- TNC LLR, Margaret Morrison"

; Monitor section
; Currently implemented commands and syntax are:

;	  <address> := low order 16 bits of Hex number
;		    := <register>
;	  <byte> := low order 8 bits of Hex number
;	  <range> := <address1>:<address2>
;		  := <address1>!<byte>
;	  <register> := #(C,A,B,D,G,X,Y,U,S,P)

;    Show contents of <address>
;	  @<address>
;    Change contents of <address> to <byte>
;	  @<address>=<byte>
;    Change contents of one address only
;	  @<address>_<byte>
;    Copy contents of <range> to location beginning with <address3>
;	  T<range> <address3>
;    Call subroutine beginning at <address>
;	  ><address>
;    List contents of <range>
;	  L<range>

	  PROG
	  SETDP     PAGE
MONBEG	  CLR	    CTO_FL		;resume normal output
	  LDA	    DBG_FL
	  BLE	    MONSTT
	  CMPA	    #BYLEN		;no re-entry during typeout
	  BEQ	    MONEND
	  PSHS	    CC,A

; Here to exit monitor

	  LDD	    #BYLEN		;protect against re-entry
	  STB	    DBG_FL
	  CLI
	  LDY	    #BYMSG		;write message & return to program
	  LBSR	    Write
MONB_1	  TST	    TYP_FL		;wait till output done
	  BNE	    MONB_1

	  PULS	    CC,A		;restore set-I, debug flag
	  STA	    DBG_FL
	  TFR	    U,S			;restore stack to original
	  LDB	    #INLEN		;transfer to input area
	  LDX	    #INLIST
	  TFR	    S,Y
	  LBSR	    Bltrn
	  LEAS	    (INLEN+OUTLEN),S
	  PULS	    A			;restore saved line-ready flag
	  STA	    LIN_FL

MONB_2	  LDB	    #1			;restore input pointers
	  SUBB	    DBG_FL
	  STB	    DBG_FL
MONEND	  RTI

; Here to enter debug from outside

MONSTT	  LDA	    LIN_FL		;save line-ready flag
	  PSHS	    A
	  LDB	    #(INLEN+OUTLEN)	;save pointers etc.
	  LEAS	    -(INLEN+OUTLEN),S
	  TFR	    S,X
	  LDY	    #INLIST
	  LBSR	    Bltrn
	  TFR	    S,U			;save stack

	  LDB	    #(INLEN+OUTLEN)
	  LDY	    #MNLIST		;transfer monitor flags &
	  LDX	    #INLIST		;pointers to I/O area
	  LBSR	    Bltrn

MONS_1	  LDB	    #1
	  SUBB	    DBG_FL
	  STB	    DBG_FL
	  CLR	    LIN_FL
	  CLI				;enable interrupts

	  CLR	    ADDRES		;zero address
	  CLR	    ADDRES+1
	  CLR	    ALT_FL		;inspect mode

MONITR	  LDA	    #MONPR		;type prompt
	  LBSR	    WRITE1
MONI_1	  LDY	    MON_BF
	  LBSR	    Getlin		;wait for typein -- read line

MPARSE	  LDA	    ,Y+			;check for key symbols
	  CMPA	    #CR			;end of line
	  BEQ	    MONDON

	  CLR	    ALT_FL
	  CMPA	    #ATSIGN		;read location
	  BEQ	    MPR_AT

	  CMPA	    #POUND		;register operations
	  LBEQ	    REGSTR
	  CMPA	    #CALL		;call subroutine
	  BEQ	    CALSUB

	  CMPA	    #TXFER		;transfer data
	  LBEQ	    TRNSFR
	  CMPA	    #LST		;list data
	  BEQ	    LSTADR

	  BRA	    MONITR		;give up & try again

SET	  EQU	    "_"
MPR_AT	  LBSR	    Dehex		;found @ -- store address
	  STD	    ADDRES
	  CLR	    ALT_FL		;not alter mode unless find
	  LDA	    ,Y+			;= in this line
	  CMPA	    #CR			;end of line
	  BEQ	    MONDON
	  CMPA	    #SET		;write one location
	  BEQ	    SETONE
	  CMPA	    #EQUAL		;write location
	  BNE	    MONITR		;give up & try again
	  LDA	    #1			;set alter mode
	  STA	    ALT_FL

SETONE	  LBSR	    Dehex		;read new byte
	  LDX	    ADDRES
	  STB	    ,X+
	  STX	    ADDRES		;save updated address
	  BRA	    MPARSE

MONDON	  TST	    ALT_FL		;end of line
	  BEQ	    MOND_1
	  LDD	    ADDRES		;display next location to change
	  LBSR	    Wrthex
	  LDA	    #EQUAL		;write = prompt
	  LBSR	    WRITE1
	  LDY	    MON_BF
	  LBSR	    Getlin		;wait for typein -- read line

MPR_EQ	  LDA	    ,Y			;get out without changing
	  CMPA	    #QUIT		;anything
	  BEQ	    MONITR
	  BRA	    SETONE

MOND_1	  LDX	    ADDRES		;write next address & contents
	  TFR	    X,D
	  LBSR	    Wrthex
	  LBSR	    WRITEBLK
	  LDA	    ,X+			;increment address
	  STX	    ADDRES
	  LBSR	    Wrtbyt
	  LBSR	    WRITEBLK		;leave a space
	  LBRA	    MONI_1

CALSUB	  LBSR	    Dehex
	  PSHS	    DP,U
	  BSR	    CALS_1		;push this PC

	  PULS	    DP
CALS_0	  TST	    TYP_FL
	  BNE	    CALS_0
	  SEI				;disable interrupts
	  PULS	    U			;get "original" stack
	  LBRA	    MONS_1		;back to monitor mode

CALS_1	  CLE				;clear entire
	  PSHS	    CC,D		;push CC, subroutine address
	  SEI
	  LBRA	    MONB_2		;return buffers, RTI

LSTADR	  BSR	    RANGE		;list range of addresses
	  STB	    BYTE
	  LDX	    ADDRES
LSTA_1	  TFR	    X,D			;set up output string
	  LBSR	    Wrthex
	  LBSR	    WRITEBLK		;leave space
	  LDA	    ,X+			;encode contents
	  LBSR	    Wrtbyt
	  LBSR	    WRITECR
	  DEC	    BYTE
	  BNE	    LSTA_1
	  STX	    ADDRES		;update address
	  LBRA	    MONITR

TRNSFR	  BSR	    RANGE		;found T -- transfer memory
	  LDA	    ,Y+			;check right format
	  CMPA	    #BLANK
	  LBNE	    MONITR

	  STB	    BYTE		;store length
	  LBSR	    Dehex		;get destination in X
	  TFR	    D,X
	  LDY	    ADDRES		;source in Y
	  LDB	    BYTE		;count in B
	  LBSR	    Bltrn
	  STX	    ADDRES		;save end destination address

	  LDY	    #DNMSG		;confirm
	  LDD	    #DNLEN
	  LBSR	    Write
	  LBRA	    MONITR

; Returns address in ADDRES, number of bytes in B

RANGE	  LBSR	    Dehex		;get first address
	  STD	    ADDRES
	  LDA	    ,Y+
	  CMPA	    #COLON
	  BEQ	    RANG_2
	  CMPA	    #POINT
	  BEQ	    RANG_1
	  LDB	    #1			;default count
	  LEAY	    -1,Y
	  RTS

RANG_1	  LBSR	    Dehex		;get count
	  RTS

RANG_2	  LBSR	    Dehex		;get second address
	  SUBD	    ADDRES		;calculate count
	  INCB
	  RTS

; Decode register addresses

	  COMN
OFFTAB	  FCB	    1,2,0,3,1,4,6,8,-1,10
REGTAB	  ASC	    "ABCGDXYUSP"

	  PROG
REGSTR	  LDA	    ,Y+			;get register code
	  LDX	    #REGTAB		;look for match
	  LDB	    #9
REGS_1	  CMPA	    B,X
	  BEQ	    REGS_2
	  DECB
	  BGE	    REGS_1
	  LBRA	    MONITR

REGS_2	  LBSR	    WRITE1		;A contains index of register
	  LDX	    #OFFTAB		;code -- look up offset
	  LDA	    B,X			;B contains offset from orig
	  BLT	    REGSTK		;stack, in U
	  CMPB	    #4			;save test for single-byte
	  PSHS	    CC
	  LEAX	    A,U
	  STX	    ADDRES		;save address of register
	  LDA	    ,Y+			;look at next byte
	  CMPA	    #EQUAL
	  BEQ	    REGS_5
	  LBSR	    WRITEBLK		;leave a space
	  LDA	    ,X+			;get actual byte
	  PULS	    CC
	  BLT	    REGS_3
	  LBSR	    Wrtbyt		;encode it
	  LDA	    ,X			;get second half
REGS_3	  LBSR	    Wrtbyt		;and encode
REGS_4	  LBSR	    WRITECR
	  LBRA	    MONITR

REGS_5	  LBSR	    Dehex		;decode new value
	  PULS	    CC
	  BLT	    REGS_6
	  STA	    ,X+			;store first half
REGS_6	  STB	    ,X			;store second half
	  LBRA	    MONITR

REGSTK	  LDA	    ,Y+			;here for stack pointer
	  CMPA	    #EQUAL
	  BEQ	    REGS_7
	  LBSR	    WRITEBLK
	  TFR	    U,D			;here to write stack pointer
	  LBSR	    Wrthex
	  BRA	    REGS_4

REGS_7	  LBSR	    Dehex		;here to change stack pointer
	  TFR	    D,U
	  LBRA	    MONITR

	  COMN
DNMSG	  ASC	    "done"
	  FCB	    CR
DNLEN	  EQU	    ($-DNMSG)

BYMSG	  FCB	    CR
	  ASC	    "Bye"
	  FCB	    CR
BYLEN	  EQU	    ($-BYMSG)

	  ENDIF
